using System;
using Dashboard.Drawing;
using Dashboard.Windowing;

namespace Dashboard.Controls
{
    public class Control : IEventListener, IDrawQueuePaintable, IDisposable
    {
        public string? Id { get; set; }
        public ClassSet Classes { get; }
        public Form? Owner { get; protected set; } = null;
        public Control? Parent { get; private set; } = null;
        public bool Disposed { get; private set; }
        public virtual DrawQueue DrawQueue => Owner?.DrawQueue ?? throw NoOwnerException;
        public virtual Box2d ClientArea { get; set; }

        public event EventHandler? Painting;
        public event EventHandler? OwnerChanged;
        public event EventHandler? ParentChanged;
        public event EventHandler? Disposing;
        public event EventHandler? Resized;

        public Control()
        {
            Classes = new ClassSet(this);
        }

        public virtual void OnPaint()
        {
            Painting?.Invoke(this, EventArgs.Empty);
        }

        public void Paint()
        {
            OnPaint();
        }

        protected void InvokeDispose(bool disposing)
        {
            if (Disposed)
                return;
            Disposed = true;

            Dispose(disposing);

            if (disposing)
                GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
        }

        public void Dispose() => InvokeDispose(true);

        public void SendEvent(EventArgs args)
        {
            throw new NotImplementedException();
        }

        internal static void SetParent(Control parent, Control child)
        {
            child.Parent = parent;
        }

        protected static Exception NoOwnerException => new Exception("No form owns this control");
    }
}
